import 'dart:async';

import 'package:camera/camera.dart';
import 'package:cloudinary_public/cloudinary_public.dart';
import 'package:flutter/material.dart';
import 'package:flutter_screenutil/flutter_screenutil.dart';
import 'package:ai_app/globals/global_colors.dart';

import 'package:ai_app/globals/global_variables.dart' as globals;
import 'package:ai_app/repositories/ai_repository.dart';
import 'package:ai_app/services/http_service.dart';
import 'package:ai_app/widgets/ai_outlined_button.dart';

class InterviewScreen extends StatefulWidget {
  @override
  _InterviewScreenState createState() {
    return _InterviewScreenState();
  }
}

void logError(String code, String? message) {
  if (message != null) {
    print('Error: $code\nError Message: $message');
  } else {
    print('Error: $code');
  }
}

class _InterviewScreenState extends State<InterviewScreen>
    with WidgetsBindingObserver, TickerProviderStateMixin {
  CameraController? controller;
  XFile? videoFile;
  VoidCallback? videoPlayerListener;
  bool isBigDoll = false;
  bool isStopButtonPressed = false;
  late HttpService httpService;
  late AiRepository aiRepository;
  late Timer _timer;
  int seconds = 0;
  int minutes = 0;
  int hours = 0;
  String secondsString = "00";
  String minutesString = "00";
  String hoursString = "00";
  final _cameraWidgetKey = GlobalKey();
  var _cameraWidgetSize;
  @override
  void initState() {
    httpService = HttpService();
    aiRepository = AiRepository(httpService);
    super.initState();
    WidgetsBinding.instance?.addObserver(this);
    _showCamera();
  }

  @override
  void dispose() {
    WidgetsBinding.instance?.removeObserver(this);
    super.dispose();
  }

  @override
  void didChangeAppLifecycleState(AppLifecycleState state) {
    final CameraController? cameraController = controller;

    if (cameraController == null || !cameraController.value.isInitialized) {
      return;
    }

    if (state == AppLifecycleState.inactive) {
      cameraController.dispose();
    } else if (state == AppLifecycleState.resumed) {
      onNewCameraSelected(cameraController.description);
    }
  }

  final GlobalKey<ScaffoldState> _scaffoldKey = GlobalKey<ScaffoldState>();

  @override
  Widget build(BuildContext context) {
    return ScreenUtilInit(
      designSize: Size(
        globals.screenWidth,
        globals.screenHeight,
      ),
      builder: () => Scaffold(
        key: _scaffoldKey,
        body: _buildContent(),
      ),
    );
  }

  _buildContent() {
    return Column(
      children: <Widget>[
        Center(
          child: _cameraPreviewWidget(),
        ),
      ],
    );
  }

  Widget _cameraPreviewWidget() {
    final CameraController? cameraController = controller;

    if (cameraController == null || !cameraController.value.isInitialized) {
      return Text(
        '',
      );
    } else {
      return _buildMain();
    }
  }

  _buildMain() {
    return isBigDoll == false
        ? Stack(
            children: [
              _buildCamera(),
              _buildDoll(),
              _captureControlRowWidget(),
              _buildAutorenewButton(),
              isStopButtonPressed == true ? _buildAfterIntrview() : Text(""),
            ],
          )
        : Stack(
            children: [
              _buildDoll(),
              _buildCamera(),
              _captureControlRowWidget(),
              _buildAutorenewButton(),
              isStopButtonPressed == true ? _buildAfterIntrview() : Text(""),
            ],
          );
  }

  _buildCamera() {
    return Padding(
      padding: isBigDoll == false
          ? const EdgeInsets.all(0)
          : EdgeInsets.only(
              top: ScreenUtil().setHeight(91),
              left: ScreenUtil().setHeight(254),
            ),
      child: Container(
        height: isBigDoll == false ? null : ScreenUtil().setHeight(123),
        width: isBigDoll == false
            ? MediaQuery.of(context).size.width
            : ScreenUtil().setWidth(97),
        child: Container(
          decoration: isBigDoll == true
              ? BoxDecoration(
                  borderRadius: BorderRadius.circular(8),
                  border: Border.all(
                    color: GlobalColors.aiGrey,
                    width: 1,
                  ),
                )
              : null,
          child: ClipRRect(
            key: _cameraWidgetKey,
            borderRadius: isBigDoll == true
                ? BorderRadius.circular(8)
                : BorderRadius.circular(0),
            child: CameraPreview(
              controller!,
            ),
          ),
        ),
      ),
    );
  }

  Widget _captureControlRowWidget() {
    final CameraController? cameraController = controller;

    return Positioned(
      bottom: ScreenUtil().setHeight(22),
      left: ScreenUtil().setWidth(62),
      child: Column(
        children: [
          Container(
            child: Column(
              children: [
                Container(
                  decoration: BoxDecoration(
                    borderRadius: BorderRadius.circular(12),
                    color: Color(0xFF6C7176).withOpacity(0.7),
                  ),
                  height: ScreenUtil().setHeight(70),
                  width: ScreenUtil().setWidth(250),
                  child: Padding(
                    padding: EdgeInsets.only(
                      top: ScreenUtil().setHeight(7),
                      bottom: ScreenUtil().setHeight(7),
                    ),
                    child: Row(
                      mainAxisAlignment: MainAxisAlignment.spaceEvenly,
                      mainAxisSize: MainAxisSize.max,
                      children: <Widget>[
                        Container(
                          height: ScreenUtil().setHeight(56),
                          width: ScreenUtil().setWidth(56),
                          decoration: BoxDecoration(
                            shape: BoxShape.circle,
                            color: GlobalColors.aiIconButtonGrey,
                          ),
                          child: IconButton(
                            icon: Icon(
                              Icons.autorenew,
                              size: ScreenUtil().setWidth(22),
                            ),
                            color: Colors.white,
                            onPressed: () {
                              stopVideoRecording().then(
                                  (value) => onVideoRecordButtonPressed());
                            },
                          ),
                        ),
                        Container(
                          height: ScreenUtil().setHeight(56),
                          width: ScreenUtil().setWidth(56),
                          decoration: BoxDecoration(
                            shape: BoxShape.circle,
                            color: GlobalColors.aiRed,
                            border: Border.all(
                              color: Colors.white,
                              width: ScreenUtil().setHeight(3),
                            ),
                          ),
                          child: IconButton(
                            icon: cameraController != null &&
                                    cameraController.value.isRecordingPaused
                                ? Icon(Icons.play_arrow)
                                : Icon(
                                    Icons.stop,
                                    size: ScreenUtil().setWidth(22),
                                  ),
                            color: Colors.white,
                            onPressed: cameraController != null &&
                                    cameraController.value.isInitialized &&
                                    cameraController.value.isRecordingVideo
                                ? (cameraController.value.isRecordingPaused)
                                    ? onResumeButtonPressed
                                    : onPauseButtonPressed
                                : () {
                                    showInSnackBar("null");
                                  },
                          ),
                        ),
                        Container(
                          height: ScreenUtil().setHeight(56),
                          width: ScreenUtil().setWidth(56),
                          decoration: BoxDecoration(
                            shape: BoxShape.circle,
                            color: GlobalColors.aiIconButtonGrey,
                          ),
                          child: IconButton(
                            icon: Icon(
                              Icons.arrow_forward,
                              size: ScreenUtil().setWidth(22),
                            ),
                            color: Colors.white,
                            onPressed: cameraController != null &&
                                    cameraController.value.isInitialized &&
                                    cameraController.value.isRecordingVideo
                                ? onStopButtonPressed
                                : () {
                                    showInSnackBar("null");
                                  },
                          ),
                        )
                      ],
                    ),
                  ),
                ),
                _buildTimer(),
              ],
            ),
          ),
        ],
      ),
    );
  }

  _showCamera() {
    final playCamera = (CameraDescription? description) {
      if (description == null) {
        return;
      }

      onNewCameraSelected(description);
    };

    if (globals.cameras.isEmpty) {
      showInSnackBar('No camera found');
    } else {
      for (CameraDescription cameraDescription in globals.cameras) {
        if (cameraDescription.lensDirection == CameraLensDirection.front) {
          playCamera(cameraDescription);

          break;
        }
      }
    }
  }

  void showInSnackBar(String message) {
    _scaffoldKey.currentState?.showSnackBar(SnackBar(content: Text(message)));
  }

  void onViewFinderTap(TapDownDetails details, BoxConstraints constraints) {
    if (controller == null) {
      return;
    }

    final CameraController cameraController = controller!;

    final offset = Offset(
      details.localPosition.dx / constraints.maxWidth,
      details.localPosition.dy / constraints.maxHeight,
    );
    cameraController.setExposurePoint(offset);
    cameraController.setFocusPoint(offset);
  }

  void onNewCameraSelected(CameraDescription cameraDescription) async {
    if (controller != null) {
      await controller!.dispose();
    }
    final CameraController cameraController = CameraController(
      cameraDescription,
      ResolutionPreset.medium,
      enableAudio: true,
      imageFormatGroup: ImageFormatGroup.jpeg,
    );
    controller = cameraController;

    cameraController.addListener(() {
      if (mounted) setState(() {});
      if (cameraController.value.hasError) {
        showInSnackBar(
            'Camera error ${cameraController.value.errorDescription}');
      }
    });

    try {
      await cameraController.initialize();
      await Future.wait([cameraController.getMinExposureOffset()]);
    } on CameraException catch (e) {
      _showCameraException(e);
    }

    if (mounted) {
      setState(() {});
    }

    if (cameraController.value.isInitialized &&
        !cameraController.value.isRecordingVideo) {
      onVideoRecordButtonPressed();
    }
  }

  void onVideoRecordButtonPressed() {
    _startTimer();
    startVideoRecording().then((_) {
      if (mounted) setState(() {});
    });
  }

  void onStopButtonPressed() {
    _stopTimer();
    setState(() {
      isStopButtonPressed = true;
    });
  }

  void onPauseButtonPressed() {
    pauseVideoRecording().then((_) {
      if (mounted) setState(() {});
    });
  }

  void onResumeButtonPressed() {
    resumeVideoRecording().then((_) {
      if (mounted) setState(() {});
    });
  }

  Future<void> startVideoRecording() async {
    final CameraController? cameraController = controller;
    if (cameraController == null || !cameraController.value.isInitialized) {
      showInSnackBar('Error: select a camera first.');
      return;
    }

    if (cameraController.value.isRecordingVideo) {
      return;
    }

    try {
      await cameraController.startVideoRecording();
    } on CameraException catch (e) {
      _showCameraException(e);
      return;
    }
  }

  Future<XFile?> stopVideoRecording() async {
    _stopTimer();
    final CameraController? cameraController = controller;
    if (cameraController == null || !cameraController.value.isRecordingVideo) {
      return null;
    }

    try {
      return cameraController.stopVideoRecording();
    } on CameraException catch (e) {
      _showCameraException(e);
      return null;
    }
  }

  Future<void> pauseVideoRecording() async {
    _timer.cancel();
    final CameraController? cameraController = controller;

    if (cameraController == null || !cameraController.value.isRecordingVideo) {
      return null;
    }

    try {
      await cameraController.pauseVideoRecording();
    } on CameraException catch (e) {
      _showCameraException(e);
      rethrow;
    }
  }

  Future<void> resumeVideoRecording() async {
    _startTimer();
    final CameraController? cameraController = controller;

    if (cameraController == null || !cameraController.value.isRecordingVideo) {
      return null;
    }

    try {
      await cameraController.resumeVideoRecording();
    } on CameraException catch (e) {
      _showCameraException(e);
      rethrow;
    }
  }

  void _showCameraException(CameraException e) {
    logError(e.code, e.description);
    showInSnackBar('Error: ${e.code}\n${e.description}');
  }

  _buildDoll() {
    return Padding(
      padding: isBigDoll == true
          ? const EdgeInsets.all(0)
          : EdgeInsets.only(
              top: ScreenUtil().setHeight(91),
              left: ScreenUtil().setHeight(254),
            ),
      child: Container(
        decoration: isBigDoll == false
            ? BoxDecoration(
                borderRadius: BorderRadius.circular(8),
                color: Colors.white,
                border: Border.all(
                  color: GlobalColors.aiGrey,
                  width: 1,
                ),
              )
            : BoxDecoration(
                color: GlobalColors.aiDollBackground,
              ),
        height: isBigDoll == true
            ? _cameraWidgetSize != null
                ? _cameraWidgetSize.height
                : null
            : ScreenUtil().setHeight(123),
        width: isBigDoll == true
            ? MediaQuery.of(context).size.width
            : ScreenUtil().setWidth(97),
        child: Padding(
          padding: isBigDoll == true
              ? EdgeInsets.only(
                  right: ScreenUtil().setHeight(125),
                )
              : EdgeInsets.all(0),
          child: Image.network(
            'https://res.cloudinary.com/feed-phonedo/image/upload/v1623825359/doll_b3aubv.png',
            fit: BoxFit.fitHeight,
          ),
        ),
      ),
    );
  }

  _buildAutorenewButton() {
    return Padding(
      padding: EdgeInsets.only(
        left: ScreenUtil().setHeight(321),
        top: ScreenUtil().setHeight(197),
      ),
      child: RawMaterialButton(
        onPressed: () {
          _getSize();
          setState(() {
            isBigDoll = !isBigDoll;
          });
        },
        fillColor: Colors.black,
        shape: CircleBorder(),
        child: Icon(
          Icons.autorenew,
          color: Colors.white,
        ),
      ),
    );
  }

  _buildAfterIntrview() {
    return Container(
      height: MediaQuery.of(context).size.height,
      width: MediaQuery.of(context).size.width,
      color: Colors.black.withOpacity(0.8),
      child: Column(
        children: [
          Padding(
            padding: EdgeInsets.only(
              top: ScreenUtil().setHeight(109),
            ),
            child: Text(
              "Hi, " + globals.firstName.toString(),
              style: TextStyle(
                color: Colors.white,
                fontSize: ScreenUtil().setSp(32),
              ),
            ),
          ),
          Padding(
            padding: EdgeInsets.only(
              top: ScreenUtil().setHeight(39),
            ),
            child: Text(
              "For some reason, you asked to stop the \ninterview, in case you do so the interview \nwill be reset and start from the beginning.",
              textAlign: TextAlign.center,
              style: TextStyle(
                color: Colors.white,
                fontSize: ScreenUtil().setSp(16),
                height: ScreenUtil().setSp(1.7),
              ),
            ),
          ),
          Padding(
            padding: EdgeInsets.only(
              top: ScreenUtil().setHeight(34),
            ),
            child: Text(
              "Are you sure?",
              style: TextStyle(
                color: Colors.white,
                fontSize: ScreenUtil().setSp(20),
              ),
            ),
          ),
          Padding(
            padding: EdgeInsets.only(
              top: ScreenUtil().setHeight(34),
            ),
            child: AiOutlinedButton(
              text: "Yes, I'm sure",
              onPressed: _stopVideo,
              isGreenButton: true,
              isBlueFont: false,
            ),
          ),
          Padding(
            padding: EdgeInsets.only(
              top: ScreenUtil().setHeight(24),
            ),
            child: AiOutlinedButton(
              text: "No, back to interview",
              onPressed: () {
                stopVideoRecording()
                    .then((value) => onVideoRecordButtonPressed());
                setState(() {
                  isStopButtonPressed = false;
                });
              },
              isGreenButton: false,
              isBlueFont: false,
            ),
          ),
        ],
      ),
    );
  }

  void _stopVideo() async {
    stopVideoRecording().then((file) async {
      if (mounted)
        setState(() {
          Navigator.pop(context);
        });
      if (file != null) {
        showInSnackBar('Video recorded to ${file.path}');
        videoFile = file;
        final cloudinary =
            CloudinaryPublic('ruthphonedo', 'hpwimk0n', cache: false);
        var videoPath = file.path;

        try {
          CloudinaryResponse response = await cloudinary.uploadFile(
            CloudinaryFile.fromFile(videoPath,
                resourceType: CloudinaryResourceType.Video),
          );

          print(response.secureUrl);
          await aiRepository.createInterview(video: response.secureUrl);
        } on CloudinaryException catch (e) {
          print(e.message);
          print(e.request);
        }
      }
    });
  }

  _buildTimer() {
    return Padding(
      padding: EdgeInsets.only(
        top: ScreenUtil().setHeight(26),
      ),
      child: Center(
        child: Container(
          decoration: BoxDecoration(
            borderRadius: BorderRadius.circular(16),
            color: GlobalColors.aiRed,
          ),
          height: ScreenUtil().setHeight(26),
          width: ScreenUtil().setWidth(76),
          child: Center(
            child: Text(
              hoursString.toString() +
                  ":" +
                  minutesString.toString() +
                  ":" +
                  secondsString.toString(),
              style: TextStyle(
                color: Colors.white,
              ),
            ),
          ),
        ),
      ),
    );
  }

  void _startTimer() {
    const oneSec = const Duration(seconds: 1);
    _timer = new Timer.periodic(
      oneSec,
      (Timer timer) => setState(
        () {
          if (seconds < 0) {
            timer.cancel();
          } else {
            seconds = seconds + 1;
            secondsString =
                seconds > 9 ? seconds.toString() : "0" + seconds.toString();
            if (seconds > 59) {
              minutes += 1;

              seconds = 0;
              secondsString = "00";
              minutesString =
                  minutes > 9 ? minutes.toString() : "0" + minutes.toString();
              if (minutes > 59) {
                hours += 1;

                minutes = 0;
                minutesString = "00";
                hoursString =
                    hours > 9 ? hours.toString() : "0" + hours.toString();
              }
            }
          }
        },
      ),
    );
  }

  _stopTimer() {
    setState(() {
      seconds = 0;
      minutes = 0;
      hours = 0;
      secondsString = "00";
      minutesString = "00";
      hoursString = "00";
    });
    _timer.cancel();
  }

  _getSize() {
    setState(() {
      _cameraWidgetSize = _cameraWidgetKey.currentContext?.size;
    });
  }
}
